package gui;

import java.awt.Color;
import java.awt.Component;
import java.util.Iterator;
import java.util.Vector;

import main.Util;

import jv.geom.PgPolygonSet;
import jv.project.PvCameraIf;
import jv.project.PvDisplayIf;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;
import jv.viewer.PvViewer;
import tsp.Node;
import tsp.TSP;
import tsp.Tour;

/** IMplementation of a JavaView display for visualizing 
 * edges of a graph as polygon sets. */
public class JvDisplay {

	/** JavaView viewer. */
	private PvViewer viewer;

	/** JavaView display. */
	private PvDisplayIf display;

	/** Polygon graph to visualize a tour of a neighborhood graph. */ 
	private PgPolygonSet graph;

	/** Nodes of the TSP instance to visualize. */
	private Vector<Node> nodes;

	/** Constructor fot TSP display. */
	public JvDisplay(TSP tsp) {

		this.viewer = new PvViewer();
		this.display = viewer.getDisplay();
		this.display.showMagnet(true);
		this.display.selectCamera(PvCameraIf.CAMERA_ORTHO_XY);
		this.display.setBackgroundColor(Color.white);

		this.nodes = tsp.getNodes();

		this.initGraph();
	}

	PgPolygonSet oldGraph;

	/** Initialziaes the polygon graph. */
	private void initGraph() {

		oldGraph = graph;

		this.graph = new PgPolygonSet(2);
		this.graph.showPolygonColors(true);

		// add the nodes of the tsp instance as JavaView-Vertices
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.elementAt(i);
			PdVector vertex = new PdVector(node.getX(), node.getY());
			vertex.setName(node.toString());
			graph.addVertex(vertex);
		}

		display.addGeometry(graph);
	}
	
	/** Adds a TSP tour to the polygon graph. */
	public void addTour( Tour _tour ) {

		initGraph();
		
		Tour tour = new Tour( _tour );
		
		tour.add( tour.at( 0 ) );

		Iterator<Node> I = tour.iterator();
		
		Node start = I.next(), end = null;
		
		while (I.hasNext()) {
			
			int j = -1;
			
			end = I.next();
			
			graph.setPolygonColor(
					j = graph.addPolygon(new PiVector(start.getID(), end
							.getID())), Color.black);

			graph.getPolygon(j).setName(
					((int) Util.cost(start, end)) + " ("
							+ ((int) Util.cost(end, start)) + ")");
			start = end;
		}

		graph.showPolygonLabels(false);
		graph.showPolygonEndArrow(false);
		graph.showPolygonColors(true);
		display.removeGeometry(oldGraph);
		display.update(graph);
	}

	/** Removes all polygon graphs added in the past. */
	public void removeAll() {
		graph.removeAllVectorFields();
	}

	/** Removes and deletes all polygn grpahs added in the past. */
	public void clear() {

		if (graph.getPolygons().length > 0) {
			graph.removePolygon(0);
			clear();
		}

		display.update(graph);
	}

	/** Adds a set of edges to the polygon graph. */
	public void addEdges(Vector<PiVector> edges) {
		
		initGraph();

		Iterator<PiVector> I = edges.iterator();

		while (I.hasNext()) 
			graph.addPolygon(I.next());

		graph.showPolygonLabels(false);
		graph.showPolygonColors(false);

		display.removeGeometry(oldGraph);
		display.update(graph);
	}

	/** Returns the internal JavaView displazy. */
	public Component getDisplay() {
		return (Component) display;
	}
}