package lk.datastructures;

import java.util.Collections;
import java.util.LinkedList;

import lk.LinKernighan;

import tsp.TSP;

/** Implemenation fo the lk ordering for our base node. */
public class LkOrdering 
{	
	private LinkedList<Move> ordering;
	
	private TSP tsp;
	
	public LkOrdering( TSP tsp,
			int base  , 
			           double delta , 
			           boolean[][] added , 
			           boolean[][] deleted ,
			           LKTour currentTour,     
			           Neighborhood neighbors,
			           boolean makMorton) 
	{  	
		this.tsp = tsp;
		
		assert neighbors != null ;
		
		ordering = new LinkedList<Move>();
		
		int nextBase = currentTour.next(base) ;
		int prevBase = currentTour.prev(base) ;
		
		// Normale Moves
		int[] neighborsOfNextBase =
			//currentTour.getNeighborsOf(nextBase)
			neighbors.getNeighborsOf(nextBase);
	  
	  	for( int i = 0; i < neighborsOfNextBase.length ; ++i) 
	  	{  	
	  		int      a = neighborsOfNextBase[i];
	  		int prev_a = currentTour.prev(a);
	  		
	  		boolean notAddedOrDeleted =
	  			! deleted[ nextBase ][ a ]  &&
	  		    ! added  [ prev_a   ][ a ]     ;
	  			
	  		/* is a  a promising Neighbor of next(base)  
	  		 * Seite 4 , (2.3)     and distinct ?      */ 

	  		
	  		if( ( delta + c( base , nextBase ) 
	  				    - c( nextBase , a ) > 0 ) &&
	  				    ( 
	  				    	   a !=     base &&  // ..... prev_a , a==base , nextBase  .....  => sinnlos
	  				      prev_a != nextBase &&  // ..... base   , prev_a==nextBase , a ..... => sinnlos
	  				      prev_a != base         // ..... prev_a == base , nextBase == a, ... => sinnlos     
	  		    		) &&
	  		            notAddedOrDeleted  ) 
	  		
	  			ordering.add( 
	  					new Move( a , prev_a , c(prev_a , a) - c( nextBase , a ) , false) );
	  		
	  	}
	  	
	  	if( LinKernighan.MAKMORTON && makMorton ) 
	  	{
	  		int[] neighborsOfBase = neighbors.getNeighborsOf(base);

	  		for( int i = 0; i < neighborsOfBase.length ; ++i) 
	  		{  	
	  			int      a = neighborsOfBase[i];
	  			int next_a = currentTour.next(a);

	  			boolean notAddedOrDeleted =
	  				! deleted[ base ][   a    ]  &&
	  				! added  [  a   ][ next_a ]     ;

	  			/* is a  a promising Neighbor of base  
	  			 * Seite 6 , vpr (2.5)     and distinct ?      */ 

	  			
	  			if( ( delta + c( base , nextBase ) 
	  					    - c( base ,   a      ) > 0 ) &&
	  					( 
	  							   a   != nextBase && // wie      a != base  
	  							next_a != prevBase && // wie prev_a != nextBase
	  							next_a != nextBase    // wie prev_a != base
	  					) &&
	  					notAddedOrDeleted  ) 

	  				ordering.add( 
	  						new Move( a , next_a , c( a , next_a ) - c( base , a ) , true ) );
	  		}
	  	}
	
	  	Collections.sort(ordering);
	}
	
	public Move nextMove() 
	{
		return ordering.poll();
	}
	
	public String toString() {
		return ordering.toString();
	}
	
	public int size() { return ordering.size(); }
	
	public boolean isEmpty() { return ordering.isEmpty(); }
	
	/** Shortcut member function for the objective. */
	private double c( int a , int b ) 
	{
		return tsp.costMatrix[a][b];
	}
}