package lk.datastructures;

import java.util.Set;
import java.util.Vector;

import jv.vecmath.PiVector;
import tsp.Node;
import tsp.TSP;
import delaunay.DelaunayTriangulation;
import delaunay.Pnt;
import delaunay.Simplex;

/** Implementation of the neighborhood graph. */
public class Neighborhood 
{
	/** Adjacency list of the neighborhood graph. */
	private Vector<int[]> neighbors ;
	
	/** Number of nodes of the graph. */
	private int n ;
	
	/** Constructs the delaunay triangulation of the given TSP instance. */
	public Neighborhood( TSP tsp ) 
	{
		this.n         = tsp.getNodes().size();
		this.neighbors = new Vector<int[]>();
		
		// delaunay triangulation 
		double maxX = tsp.getMaxX();
    	double maxY = tsp.getMaxY();
    	
    	double max = maxX < maxY ? maxY : maxX ;
    	
    	// big triangle
    	
    	Simplex<Pnt> tri = 
    		new Simplex<Pnt>(new Pnt(-max,-0.5), new Pnt(2 * max,-0.5), new Pnt(-0.5,2 * max));
    	
    	DelaunayTriangulation dt = new DelaunayTriangulation(tri);

    	Vector<Node> points = new Vector<Node>( tsp.getNodes() );
    	
    	for (Node point : points) 
    	{
    		Pnt p = new Pnt( point.getX() , point.getY() );
    		p .id( point.getID() );
    		dt.delaunayPlace(p);
		}
    	
    	Vector<Vector<Integer>> triangulation = 
    		new Vector<Vector<Integer>> () ;
    	
    	for(int s = 0 ; s < points.size(); s++) 
    	{
    		triangulation.add(new Vector<Integer>());
    	}
    	
    	for (Simplex<Pnt> triangle: dt) 
    	{
    		for (Set<Pnt> edge: triangle.facets()) 
    		{
    			int [] kante = new int[2]; int i = 0 ;
    			for (Pnt pnt : edge) kante[i++] = pnt.id() ;
    			
    			if( kante[0] != -1 && kante[1] != -1) 
    			{
    				int s = kante[0];
    				int t = kante[1]; assert s != t ;
    				
    				if (triangulation.elementAt(s).contains(t) || 
    				    triangulation.elementAt(t).contains(s)) continue ;
    				
    				triangulation.elementAt(s).add(t) ;
    				triangulation.elementAt(t).add(s) ;
    			}
    		}
    	}
    	
    	// construct the edges for visualization 
    	this.visEdges = new Vector<PiVector>();
    	
    	assert triangulation.size() == n : triangulation.size() + "\t"+n;
    	
    	for (int s = 0; s < triangulation.size(); s++) 
    	{
    		Vector<Integer> adjazent = 
    			triangulation.elementAt(s);
    		
    		int[] neighbors = new int[adjazent.size()];
    		
    		for(int j = 0; j < adjazent.size(); ++j  )
    		{
    			neighbors[j] = adjazent.elementAt(j);
    			visEdges.add(new PiVector( s, neighbors[j]));
    			
    		}
    		
    		this.neighbors.add(neighbors);
		}   
	}

	/** Vector of edges to visualize. */
	private Vector<PiVector> visEdges;
	
	/** Returns the vector of edges to visualize. */
	public  Vector<PiVector> getEdges() 
	{
		return visEdges;
	}
	
	/** Returns the nodes which are neighbors of the node with the given id. */
	public int[] getNeighborsOf( int id ) 
	{
		return neighbors.elementAt( id );
	}
}