package main;

import gui.TSPGui;

import lk.LinKernighan;
import lk.datastructures.Neighborhood;

import tsp.Node;
import tsp.TSP;
import tsp.Tour;

public class Main {

	public static void main(String[] a) throws InterruptedException {

		// if the program was not called with a file name argument,
		// print usage and exit with status 1

		// name of the input file
		String inputFile = null;

		// sleep time in seconds
		int sleepTime = 500;

		// switch over the number of arguments
		switch (a.length) {

		case 2:
			
			try {
				sleepTime = Integer.parseInt(a[1]) * 1000;
			} catch (NumberFormatException e) {
				System.err.println("could not parse sleep time \'" + a[1]
						+ "\'");
				Util.printUsageAndExit();
			}

			System.out.printf("sleep time was set to %d seconds\n",
					sleepTime / 1000);

		case 1:
			inputFile = a[0];
			break;

		default:
			Util.printUsageAndExit();
		}

		// create a new TSP instance
		TSP tsp = new TSP(inputFile);

		// visualization
		TSP.gui = new TSPGui(tsp);

		// most simple construction heuristic
		Tour tour = new Tour();

		for (Node node : tsp.getNodes()) {
			tour.add(node);
		}

		double time = System.currentTimeMillis();

		// compute delaunay triangulation
		Neighborhood neighborhood = new Neighborhood(tsp);

		// visualize the edges of the triangulartion
		TSP.gui.getJvDisp().addEdges(neighborhood.getEdges());

		System.out.printf(
				"time for contruction delaunay neighborhood = %.2f sec\n",
				(System.currentTimeMillis() - time) / 1000);
		
		// Special debugging switch: if the sleep has been set to a negative value
		// we exit the program. That is useful to have an start-read-self-test.
		if( sleepTime < 0 )
		{
			System.out.printf( "exiting because sleep time has been set to %d\n", sleepTime/1000 );
			System.exit( 0 );
		}

		Thread.sleep(sleepTime);

		LinKernighan jLK = new LinKernighan(tour, tsp, neighborhood);

		// visualize the initial tour
		TSP.gui.getJvDisp().addTour( tour );

		System.out.println("cost of initial tour       = " + tour.cost());

		Thread.sleep(sleepTime);

		// run the Lin-Kernighan heuristic 
		tour = jLK.run();

		System.out.println("cost of local optimal tour = " + tour.cost());
	}
}
