package tsp;

import java.io.Serializable;

/** Represents a node of a TSP instance in pure data representation, i.e., it
 * does not implement any algorithmic aspects.*/
public class Node implements Serializable {

	/** Default serial version id. */
	private static final long serialVersionUID = 1L;

	/** Id of this node. */
	private int id;
	
	/** Name of this node. */
	private String name;

	/** x-coordiante of this node. */
	private double x;

	/** y-coordiante of this node. */
	private double y;

	/** Constructor. */
	public Node(int id, String name, double x, double y) {
		this.id = id;
		this.name = name;
		this.setX(x);
		this.setY(y);
	}

	/** Returns the x coordinate of this node. */
	public double getX() {
		return x;
	}

	/** Returns the y coordinate of this node. */
	public double getY() {
		return y;
	}
	
	/** Returns the name of this node. */
	public String getName() {
		return name;
	}

	/** Returns the id of this node. */
	public int getID() {
		return id;
	}

	/** Returns a string representation of this node. */
	public String toString() {
		return getName() + " (" + getX() + "," + getY() + ")";
	}

	/** Returns a copy of this node. */
	public Node clone() {
		return new Node(id, name, getX(), getY());
	}

	/**
	 * @param x the x to set
	 */

	public void setX(double x) {
		this.x = x;
	}

	/**
	 * @param y the y to set
	 */
	public void setY(double y) {
		this.y = y;
	}
}